/* Demonstration program for matrix operations related to
   matrix multiplication with n**2 processors */

#include <stdio.h>

/* Maximum size for demonstration */
#define N 10

int A[N][N], B[N][N];  /* Represent current arrangements of arrays */

/* A[i,j] = k means current value is a[i,k], -1 means garbage */
/* B[i,j] = k means current value is b[k,j], -1 means garbage */

int M[N][N];           /* Defines cells for multiplication */

int W;                 /* Controls wrap-around during shifts */

/* Make matrices of size n by n */

void start(n)
 int n;
 {int i, j;
  for ( i = 0; i < n; i++ )
   for ( j = 0; j < n; j++ )
    { A[i][j] = j;  B[i][j] = i; };
 }

/* Draw horizontal line for n cells */

horiz(n)
 int n;
 {int k;
  for ( k = 0; k < n; k++ ) printf("+-----");
  printf("+\n");
 }

/* Display item i, j.  Flags product cell if m  */

ditem(i, j, m)
 int i, j, m;
 {if (i >= 0 && j >= 0)
       {if (m) printf("|*%1d %1d*", i, j);
          else printf("| %1d %1d ", i, j);}
  else printf("| --- ");
 }

/* Display current status for size n, flag products if m */

display(n, m)
 int n, m;
 {int i, j;
  for ( i = 0; i < n; i++ )
   {horiz(n);
    for ( j = 0; j < n; j++ ) ditem(i, A[i][j], m && M[i][j]);
    printf("|\n");
    for ( j = 0; j < n; j++ ) ditem(B[i][j], j, m && M[i][j]);
    printf("|\n");
   };
  horiz(n);
 }

/* Shift up 1 position in cols a - b */

up(n, a, b)
 int n, a, b;
 {int i, j, r;
  for ( j = a; j <= b; j++ )
   {r = (W ? B[0][j] : -1);
    for ( i = 0; i < n-1; i++ ) B[i][j] = B[i+1][j];
    B[n-1][j] = r;
   }
 }

/* Shift down 1 position in cols a - b */

dn(n, a, b)
 int n, a, b;
 {int i, j, r;
  for ( j = a; j <= b; j++ )
   {r = (W ? B[n-1][j] : -1);
    for ( i = n-1; i > 0; i-- ) B[i][j] = B[i-1][j];
    B[0][j] = r;
   }
 }

/* Shift left 1 position in rows a - b */

lf(n, a, b)
 int n, a, b;
 {int i, j, r;
  for ( i = a; i <= b; i++ )
   {r = (W ? A[i][0] : -1);
    for ( j = 0; j < n-1; j++ ) A[i][j] = A[i][j+1];
    A[i][n-1] = r;
   }
 }

/* Shift right 1 position in rows a - b */

rt(n, a, b)
 int n, a, b;
 {int i, j, r;
  for ( i = a; i <= b; i++ )
   {r = (W ? A[i][n-1] : -1);
    for ( j = n-1; j > 0; j-- ) A[i][j] = A[i][j-1];
    A[i][0] = r;
   }
 }

/* Load all mult flags with f */

loadm(n, f)
 int n, f;
 {int i, j;
  for ( i = 0; i < n; i++ )
   for ( j = 0; j < n; j++ ) M[i][j] = f;
 }

/* Set mult flags on diag i + j = d to f */

diagm(n, d, f)
 int n, d, f;
 {int i, j;
  for ( i = 0, j = d; j >= 0; i++, j-- ) M[i][j] = f;
 }

/* Set mult flags from [d,d] to borders b to f */

chevm(n, d, b, f)
 int n, d, b, f;
 {int k, inc, lim;
  if (b) { inc = 1; lim = n; } else { inc = -1; lim = 1; };
  M[d][d] = f;
  for ( k = d+inc; inc*k < lim; k += inc ) M[d][k] = M[k][d] = f;
 }

/* Pause for signal from user */

pause()
 {char c[2];
  printf("Type any character to proceed>");  scanf("%1s", c);
 } 

/* Demonstrate multiply with grid */

demog(n)
 int n;
 {int k, lst, w;
  lst = n-1;  w = W;  W = 0;
  loadm(n, 0);  start(n);  display(n, 0);
  for ( k = lst; k > 0; k--)
   {lf(n, k, lst);  up(n, k, lst);  diagm(n, k, 0);  chevm(n, k-1, n, 1);
    pause();  display(n, 1); };
  for ( k = lst; k > 0; k--)
   {lf(n, 0, k);  up(n, 0, k);  diagm(n, k, 0);
    pause();  display(n, 1); };
  loadm(n, 0);  start(n);  pause();  display(n, 0);
  for ( k = 0; k < lst; k++)
   {rt(n, 0, k);  dn(n, 0, k);  diagm(n, k, 0);  chevm(n, k+1, 0, 1);
    pause();  display(n, 1); };
  for ( k = 0; k < lst; k++)
   {rt(n, k, lst);  dn(n, k, lst);  diagm(n, k, 0);
    pause();  display(n, 1); };
  W = w;
 }

/* Demonstrate multiply with cycles */

democ(n)
 int n;
 {int k, l, m, lst, w;
  lst = n-1;  w = W;  W = 1;  m = n/2;
  loadm(n, 1);  start(n);  display(n, 0);
  for ( k = 1, l = lst; k <= m; k++, l-- )
   {up(n, k, m);  lf(n, k, m);
    if ( m < l ) { rt(n, m+1, l); dn(n, m+1, l); };
    pause();  display(n, 0);
   };
  pause();  display(n, 1);
  for ( k = 1; k < n; k++ )
   {rt(n, 0, lst);  dn(n, 0, lst);  pause();  display(n, 1); };
  W = w;
 }
 
/* Read and check interval spec.  Returns non-zero if OK. */

int getab(n, p, q)
 int n, *p, *q;
 {int a, b;
  scanf("%d%d", &a, &b);
  if ( 0 <= a && a <= b && b < n ) {*p = a;  *q = b;  return(1); }
      else {printf("Bad interval\n");  return(0); }
 }

/* Test program.  Commands are
   m <n>  - initialize matrix of size n by n
   p      - print current status
   o      - return to original status
   l r u d <a> <b>
          - shift {left right up down} in {rows cols} a through b
   G      - grid demo with current matrix
   C      - cyclic demo with current matrix
   w <n>  - set wrap-around to n
   q      - quit
*/

void main()
 { char c[2];  int n, a, b;
   W = 0;
   while ( printf("matrix>"), scanf("%1s", c), c[0] != 'q' )
    switch (c[0])
     { case 'm':  scanf("%d", &n);
       case 'o':  if (n < 0 || n > N) printf("Bad n\n");
                    else { start(n); /* display(n); */ };
                  break;
       case 'p':  display(n, 0);  break;
       case 'l':  if ( getab(n, &a, &b) ) lf(n,a,b);  break; 
       case 'r':  if ( getab(n, &a, &b) ) rt(n,a,b);  break; 
       case 'u':  if ( getab(n, &a, &b) ) up(n,a,b);  break; 
       case 'd':  if ( getab(n, &a, &b) ) dn(n,a,b);  break;
       case 'G':  demog(n);  break;
       case 'C':  democ(n);  break;
       case 'w':  scanf("%d", &W);  break; 
       default:   printf("Bad input - ignored\n");  break;
     };
 }
